@@ -121,6 +121,9 @@ pub struct Options {
     pub mounts: bool,
 
     pub color_scale: ColorScaleOptions,
+
+    /// Whether to drill down into symbolic links that point to directories
+    pub follow_links: bool,
 }
 
 pub struct Render<'a> {
@@ -291,8 +294,15 @@ impl<'a> Render<'a> {
                     .map(|t| t.row_for_file(file, self.show_xattr_hint(file), color_scale_info));
 
                 let mut dir = None;
+                let follow_links = self.opts.follow_links;
                 if let Some(r) = self.recurse {
-                    if file.is_directory() && r.tree && !r.is_too_deep(depth.0) {
+                    if (if follow_links {
+                        file.points_to_directory()
+                    } else {
+                        file.is_directory()
+                    }) && r.tree
+                        && !r.is_too_deep(depth.0)
+                    {
                         trace!("matching on to_dir");
                         match file.to_dir() {
                             Ok(d) => {
@@ -320,7 +330,7 @@ impl<'a> Render<'a> {
 
         for (tree_params, egg) in depth.iterate_over(file_eggs.into_iter()) {
             let mut files = Vec::new();
-            let mut errors = egg.errors;
+            let errors = egg.errors;
 
             if let (Some(ref mut t), Some(row)) = (table.as_mut(), egg.table_row.as_ref()) {
                 t.add_widths(row);
@@ -352,14 +362,7 @@ impl<'a> Render<'a> {
                     egg.file.deref_links,
                     egg.file.is_recursive_size(),
                 ) {
-                    match file_to_add {
-                        Ok(f) => {
-                            files.push(f);
-                        }
-                        Err((path, e)) => {
-                            errors.push((e, Some(path)));
-                        }
-                    }
+                    files.push(file_to_add);
                 }
 
                 self.filter
